from PyQt5.QtWidgets import QDialog, QVBoxLayout, QLabel, QLineEdit, QPushButton, QMessageBox
from qgis.PyQt.QtCore import Qt
import os
import json

from .license_manager import LicenseManager
from .license_input_dialog import LicenseInputDialog
from .chibansearch import chibansearch

CONFIG_FILE = os.path.join(os.path.dirname(__file__), "config.json")
LOCAL_LICENSE_FILE = os.path.join(os.path.dirname(__file__), "local_license.json")

def classFactory(iface):
    # ローカルのライセンスファイルをチェック
    if os.path.exists(LOCAL_LICENSE_FILE):
        with open(LOCAL_LICENSE_FILE, 'r') as f:
            local_license = json.load(f)
        
        # ローカルファイルに保存されているライセンスキーを検証
        license_manager = LicenseManager(CONFIG_FILE)
        if license_manager.validate_license(local_license['license_key']) and license_manager.check_expiry():
            return chibansearch(iface)  # プラグインを起動
    
    # ライセンス認証ダイアログの表示
    input_dialog = LicenseInputDialog(CONFIG_FILE)
    if input_dialog.exec_() == LicenseInputDialog.Accepted:
        license_key = input_dialog.lineEdit.text().strip()

        # ライセンスマネージャーの初期化
        license_manager = LicenseManager(CONFIG_FILE)

        # ライセンスの検証
        if license_manager.validate_license(license_key) and license_manager.check_expiry():
            # ローカルライセンスファイルに保存
            with open(LOCAL_LICENSE_FILE, 'w') as f:
                json.dump({"license_key": license_key}, f)
            
            return chibansearch(iface)  # プラグインを起動
        else:
            raise RuntimeError("ライセンスが無効または期限切れです。サポートにお問い合わせください。")
    else:
        raise RuntimeError("ライセンス入力ダイアログがキャンセルされました。プラグインをロードできません。")
