# -*- coding: utf-8 -*-
"""
/***************************************************************************
 chibansearch
                                 A QGIS plugin
                                 cording by masahiro ashizawa
                                 e-mail:info@office-ashizawa.com
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/
"""

from qgis.utils import iface
from qgis.core import *
from qgis.core import QgsVectorLayer
from qgis.core import QgsProject
from qgis.core import Qgis
from qgis.PyQt.QtCore import QSettings, QTranslator, QCoreApplication
from qgis.PyQt.QtGui import QIcon
from qgis.PyQt.QtWidgets import QAction
from PyQt5.QtGui import *
from PyQt5.QtCore import *
from PyQt5.QtWidgets import *
from .resources import *
from PyQt5.QtPrintSupport import QPrintDialog, QPrinter

import os
import os.path
import csv
import math

from .chibansearch_dockwidget import chibansearchDockWidget


class chibansearch:
    """QGIS Plugin Implementation."""

    def __init__(self, iface):
        """Constructor.

        :param iface: An interface instance that will be passed to this class
            which provides the hook by which you can manipulate the QGIS
            application at run time.
        :type iface: QgsInterface
        """
        # Save reference to the QGIS interface
        self.iface = iface

        # initialize plugin directory
        self.plugin_dir = os.path.dirname(__file__)

        # initialize locale
        locale = QSettings().value('locale/userLocale')[0:2]
        locale_path = os.path.join(
            self.plugin_dir,
            'i18n',
            'chibansearch_{}.qm'.format(locale))

        if os.path.exists(locale_path):
            self.translator = QTranslator()
            self.translator.load(locale_path)
            QCoreApplication.installTranslator(self.translator)

        # Declare instance attributes
        self.actions = []
        self.menu = self.tr(u'&chibansearch')
        # TODO: We are going to let the user set this up in a future iteration
        self.toolbar = self.iface.addToolBar(u'chibansearch')
        self.toolbar.setObjectName(u'chibansearch')

        #print "** INITIALIZING chibansearch"

        self.pluginIsActive = False
        self.dockwidget = None


    # noinspection PyMethodMayBeStatic
    def tr(self, message):
        """Get the translation for a string using Qt translation API.

        We implement this ourselves since we do not inherit QObject.

        :param message: String for translation.
        :type message: str, QString

        :returns: Translated version of message.
        :rtype: QString
        """
        # noinspection PyTypeChecker,PyArgumentList,PyCallByClass
        return QCoreApplication.translate('chibansearch', message)


    def add_action(
        self,
        icon_path,
        text,
        callback,
        enabled_flag=True,
        add_to_menu=True,
        add_to_toolbar=True,
        status_tip=None,
        whats_this=None,
        parent=None):

        icon = QIcon(icon_path)
        action = QAction(icon, text, parent)
        action.triggered.connect(callback)
        action.setEnabled(enabled_flag)

        if status_tip is not None:
            action.setStatusTip(status_tip)

        if whats_this is not None:
            action.setWhatsThis(whats_this)

        if add_to_toolbar:
            self.toolbar.addAction(action)

        if add_to_menu:
            self.iface.addPluginToMenu(
                self.menu,
                action)

        self.actions.append(action)

        return action


    def initGui(self):
        """Create the menu entries and toolbar icons inside the QGIS GUI."""

        self.add_action(
            icon_path = os.path.dirname(__file__) + '/mapicon.png',
            text=self.tr(u'chibansearch'),
            callback=self.run,
            parent=self.iface.mainWindow())

    #--------------------------------------------------------------------------

    def onClosePlugin(self):
        """Cleanup necessary items here when plugin dockwidget is closed"""

        #print "** CLOSING chibansearch"

        # disconnects
        self.dockwidget.closingPlugin.disconnect(self.onClosePlugin)
        self.pluginIsActive = False


    def unload(self):
        """Removes the plugin menu item and icon from QGIS GUI."""

        #print "** UNLOAD chibansearch"

        for action in self.actions:
            self.iface.removePluginMenu(
                self.tr(u'&chibansearch'),
                action)
            self.iface.removeToolBarIcon(action)
        # remove the toolbar
        del self.toolbar

    #--------------------------------------------------------------------------

    def run(self):
        if not self.pluginIsActive:
            self.pluginIsActive = True

            if self.dockwidget == None:
                self.dockwidget = chibansearchDockWidget()
                self.first_flg = True

            self.dockwidget.closingPlugin.connect(self.onClosePlugin)
            self.dlgUpdate()

            self.iface.addDockWidget(Qt.TopDockWidgetArea, self.dockwidget)
            self.dockwidget.show()


    def dlgUpdate(self):
        if self.first_flg:
            self.dockwidget.btnSet.clicked.connect(self.initOoazaCombobox)
            self.dockwidget.btnSearchChiban.clicked.connect(self.SearchChibanClicked)
            self.dockwidget.captureButton.clicked.connect(self.captureButtonPushed)
            self.first_flg = False


    def captureButtonPushed(self):
        self.previous_map_tool = self.iface.mapCanvas().mapTool()
        map = iface.mapCanvas()
        QApplication.clipboard().setImage(QImage(QGraphicsView.grab(map)))
        iface.messageBar().pushMessage("キャプチャ", "：クリップボードに地図画面を保存しました", level=Qgis.Info)


    def initOoazaCombobox(self):
        # 現在選択されているレイヤを取得
        active_layer = self.iface.activeLayer()
        if not active_layer:
            return  # アクティブなレイヤがない場合は何もしない
                
        cmbname = active_layer.name()
        layers = QgsProject.instance().mapLayersByName(cmbname)

        comboBoxOoaza = self.dockwidget.comboBoxOoaza
        comboBoxOoaza.clear()
        comboBoxOoaza.setEditable(True)

        if len(layers) != 0:
            layer = layers[0]
            self.iface.setActiveLayer(layer)

            field_name = "大字名"  # ユニーク値を取得するフィールド名

            field_index = layer.fields().lookupField(field_name)  # フィールド名からインデックス番号を取得
            unique_values = layer.uniqueValues(field_index)  # インデックス番号を使用してユニークな値を取得

            # NULL値を除外する
            unique_values = [value for value in unique_values if value is not None]  # NULL値を除外する

            # ソートする
            unique_values.sort()

            comboBoxOoaza.clear()
            comboBoxOoaza.addItems([str(value) for value in unique_values])
            comboBoxOoaza.setCurrentIndex(comboBoxOoaza.count() - 1)

            iface.messageBar().pushMessage("処理完了", "：大字フィルタ処理が完了しました", level=Qgis.Info)


    def SearchChibanClicked(self, index):

        # 現在選択されているレイヤを取得
        active_layer = self.iface.activeLayer()
        if not active_layer:
            return  # アクティブなレイヤがない場合は何もしない

        cmbname = active_layer.name()
        ooazavalue = self.dockwidget.comboBoxOoaza.currentText()
        chibanvalue = self.dockwidget.lineEditChiban.text()
        edabanvalue = self.dockwidget.lineEditEdaban.text()

        layers = QgsProject.instance().mapLayersByName(cmbname)
        if len(layers) != 0:
            layer = layers[0]
            iface.setActiveLayer(layer)

            if edabanvalue != "":
                query = "\"大字名\" = '{}' AND \"本番\" = '{}' AND \"枝番\" = '{}'".format(ooazavalue, chibanvalue, edabanvalue)
            else:
                query = "\"大字名\" = '{}' AND \"本番\" = '{}'".format(ooazavalue, chibanvalue)

            expression = QgsExpression(query)
            features = layer.getFeatures(QgsFeatureRequest(expression))
            ids = [feature.id() for feature in features]
            layer.selectByIds(ids)

            layer = iface.activeLayer()  # アクティブなレイヤーを取得する
            selected_features = layer.selectedFeatures()  # 選択されたフィーチャを取得する

            if len(selected_features) > 0:
                feature = selected_features[0]  # 最初に選択されたフィーチャを取得する

                fields = layer.fields()
                
                iface.mapCanvas().zoomToSelected()
                # 選択されたコンボボックスの値を保持して、再設定する
                current_ooaza_value = self.dockwidget.comboBoxOoaza.currentText()
                self.initOoazaCombobox()
                self.dockwidget.comboBoxOoaza.setCurrentText(current_ooaza_value)
                iface.messageBar().pushMessage("処理完了", "：検索結果が表示されました", level=Qgis.Info)
            else:
                iface.messageBar().pushMessage("処理完了", "：指定された地番が存在しません", level=Qgis.Info)
                

