import datetime
import json
import hashlib

class LicenseManager:
    def __init__(self, config_file):
        self.config_file = config_file
        self.load_config()

    def load_config(self):
        with open(self.config_file, 'r') as f:
            self.config = json.load(f)

    def save_config(self):
        with open(self.config_file, 'w') as f:
            json.dump(self.config, f, indent=4)

    def hash_password(self, password):
        return hashlib.md5(password.encode()).hexdigest()

    def validate_license(self, license_key):
        license_hash = self.hash_password(license_key)
        stored_hashes = self.config['License']['license_keys']
        if license_hash in stored_hashes:
            return True
        else:
            return False

    def check_expiry(self):
        current_date = datetime.datetime.now()
        expiry_date_str = self.config['License']['expiry_date']
        expiry_date = datetime.datetime.strptime(expiry_date_str, "%Y-%m-%d")
        
        if current_date < expiry_date:
            return True
        else:
            return False
